// BtBattleship.java

package ch.aplu.btbattleship;

import ch.aplu.android.*;
import ch.aplu.android.bluetooth.*;
import android.bluetooth.*;
import android.graphics.Color;
import ch.aplu.util.Monitor;

public class BtBattleship extends GameGrid
  implements GGNavigationListener, GGTouchListener, BluetoothPeerListener
{
  private enum State
  {
    DEPLOY, CONNECTING, MY_SHOOT, ENEMY_SHOOT,
    GAME_OVER, CONNECTION_LOST
  }

  private interface Command
  {
    int disconnect = -3;
    int ready = -2;
    int restart = -1;
    int miss = 0;
    int hit = 1;
    int sunk = 2;
    int all_sunk = 3;
  }

  private volatile State state = State.DEPLOY;
  private String msgMyShoot = "Click a cell to fire";
  private String msgEnemyShoot = "Please wait enemy bomb";
  private Ship[] fleet = new Ship[3];
  private Location currentLoc;
  private final String serviceName = "Battleship";
  private BluetoothPeer bp;
  private GGStatusBar status;
  private static Marker marker;
  private final Location hideLocation = new Location(-1, 1);
  private Ship activeShip = null;
  private final int DKGREEN = Color.rgb(0, 80, 0);
  private boolean isGameLost;
  private boolean isEnemyReady;

  public BtBattleship()
  {
    super(8, 5, cellZoom(100), BLACK, null, false);
    setScreenOrientation(LANDSCAPE);
    setNbRotSprites(4);
    status = addStatusBar(30);
  }

  public void main()
  {
    status.setRefreshEnabled(true);
    getBg().clear(BLUE);
    setSimulationPeriod(50);

    init();
    addTouchListener(this, GGTouch.press | GGTouch.drag | GGTouch.release);
    addNavigationListener(this);

    Monitor.putSleep();  // Wait until fleet is deployed
    connect();
  }

  private void init()
  {
    removeAllActors();
    status.setText("Drag at tail. [MENU]:rotate. [BACK]:start",
      DKGRAY);

    fleet[0] = new Carrier();
    fleet[1] = new Cruiser();
    fleet[2] = new Submarine();

    for (int i = 0; i < fleet.length; i++)
      addActor(fleet[i], new Location(0, 2 * i));

    marker = new Marker();
    addActor(marker, hideLocation);  // Outside
    setTouchEnabled(true);
  }

  public boolean touchEvent(GGTouch touch)
  {
    if (state == State.CONNECTION_LOST)
      return true;

    currentLoc = toLocation(touch.getX(), touch.getY());
    if (!isInGrid(currentLoc))
      return false;
    if (state == State.DEPLOY)
    {
      switch (touch.getEvent())
      {
        case GGTouch.press:
          boolean isTapOnShip = false;
          for (Ship ship : fleet)
          {
            // Check if one of the ships are touched
            for (int i = 0; i < ship.size; i++)
            {
              if (ship.cells[i].equals(currentLoc))
              {
                marker.setLocation(ship.getLocation());
                activeShip = ship;
                isTapOnShip = true;
                break;
              }
            }
          }
          if (!isTapOnShip)  // Tap outside fleet
          {
            marker.setLocation(hideLocation);
            activeShip = null;
          }
          break;
        case GGTouch.drag:
          if (activeShip != null && activeShip.advance(currentLoc, 0))
          {
            activeShip.setLocation(currentLoc);
            marker.setLocation(currentLoc);
            activeShip.updateCells(currentLoc);
          }
          break;
      }
      refresh();
    }

    if (state == State.MY_SHOOT)
    {
      setTouchEnabled(false);
      int[] data =
      {
        currentLoc.x, currentLoc.y
      };
      bp.sendDataBlock(data);
    }

    return true;
  }

  public void receiveDataBlock(int[] data)
  {
    // Command dispatcher
    switch (data[0])
    {
      case Command.disconnect:
        notifyConnection(null, false);
        return;
      case Command.restart:
        if (isGameLost)
        {
          state = State.MY_SHOOT;
          status.setText(msgMyShoot, DKGREEN);
        }
        else
        {
          state = State.ENEMY_SHOOT;
          status.setText(msgEnemyShoot, RED);
        }
        return;
      case Command.ready:
        status.setText("Enemy ready to play. [BACK]:start", DKGREEN);
        isEnemyReady = true;
        return;
    }

    // We show the feedback of our shoot
    if (state == State.MY_SHOOT)
    {
      showFeedback(data[0]);
      if (state != State.GAME_OVER)
      {
        state = State.ENEMY_SHOOT;
        status.setText(msgEnemyShoot, RED);
      }
      return;
    }

    // We got a bomb and handle the impact
    Location loc = new Location(data[0], data[1]);
    int feedback = showImpact(loc);
    sendCommand(feedback);
    if (state != State.GAME_OVER)
    {
      state = State.MY_SHOOT;
      status.setText(msgMyShoot, DKGREEN);
      setTouchEnabled(true);
    }
  }

  private int showImpact(Location loc)
  {
    for (Actor a : getActors(Ship.class))
    {
      String s = ((Ship)a).hit(loc);
      if (s.equals("hit"))
        return Command.hit;
      if (s.equals("sunk"))
        return Command.sunk;
      if (s.equals("allSunk"))
      {
        state = State.GAME_OVER;
        removeAllActors();
        addActor(new Actor("gameover"), new Location(3, 1));
        addActor(new Actor("allsunk"), new Location(3, 3));
        status.setText("Game over. [BACK]:play again", DKGREEN);
        isGameLost = true;
        isEnemyReady = false;
        return Command.all_sunk;
      }
    }
    // miss
    addActor(new Water(), loc);
    refresh();
    return 0;
  }

  private void showFeedback(int command)
  {
    switch (command)
    {
      case Command.miss: // miss
        addActorNoRefresh(new Actor("miss"), currentLoc);
        break;
      case Command.hit: // hit
        addActorNoRefresh(new Actor("hit"), currentLoc);
        break;
      case Command.sunk: // sunk
        addActorNoRefresh(new Actor("sunk"), currentLoc);
        break;
      case Command.all_sunk: // allsunk
        state = State.GAME_OVER;
        removeAllActors();
        addActorNoRefresh(new Actor("gameover"), new Location(3, 1));
        addActorNoRefresh(new Actor("winner"), new Location(3, 3));
        status.setText("Game over. [BACK]:Play again", DKGREEN);
        isGameLost = false;
        setTouchEnabled(false);
        isEnemyReady = false;
        break;
    }
  }

  private void connect()
  {
    String partnerName = askName();
    BluetoothDevice partnerDevice = searchDevice(partnerName);
    if (partnerDevice != null)
    {
      status.setText("Connecting to '" + partnerName + "'..");
      bp = new BluetoothPeer(this, partnerDevice, serviceName, this, true);
      if (bp.isConnected())
      {
        status.setText("Connection OK. " + msgMyShoot, DKGREEN);
        state = State.MY_SHOOT;  // Client has first move
      }
      else
        status.setText("'" + partnerName + "' not found.Server '"
          + BluetoothDiscovery.getBluetoothAdapter(this).getName() + "' waiting..",
          DKGREEN);
    }
  }

  public void notifyConnection(BluetoothDevice device, boolean connected)
  {
    if (connected)
    {
      status.setText("Connection OK. Wait for enemy's shoot", RED);
      state = State.ENEMY_SHOOT;  // Client has first move
    }
    else
    {
      status.setText("Connection lost. Press [HOME]", RED);
      state = State.CONNECTION_LOST;
    }
  }

  private String askName()
  {
    GGPreferences prefs = new GGPreferences(this);
    String oldName = prefs.retrieveString("BluetoothName");
    String newName = null;
    while (newName == null)
    {
      newName = GGInputDialog.show(this, "Search Device",
        "Enter Bluetooth Name",
        oldName == null ? "" : oldName);
    }
    prefs.storeString("BluetoothName", newName);
    return newName;
  }

  /*
   * Searches for device, first in set of paired devices than
   * by a Bluetooth discovery. Returns null, if device not found.
   */
  private BluetoothDevice searchDevice(String btName)
  {
    BluetoothAdapter adapter =
      BluetoothDiscovery.getBluetoothAdapter(this);
    if (adapter == null)
    {
      status.setText("Bluetooth not supported", RED);  // e.g. in emulator
      return null;
    }
    if (!adapter.isEnabled())
    {
      requestBluetoothEnable();
      if (!adapter.isEnabled())
      {
        status.setText("Can' enable Bluetooth", RED);
        return null;
      }
    }
    status.setText("Searching for paired device '" + btName + "'..");
    BluetoothDiscovery bd = new BluetoothDiscovery(this);
    BluetoothDevice device = bd.getPairedDevice(btName);
    if (device != null)
      status.setText("Paired device '" + btName + "' found.");
    else
      status.setText("Please pair with device '" + btName + "'");
    return device;
  }

  public void navigationEvent(GGNavigationEvent event)
  {
    if (state == State.CONNECTION_LOST)
      return;

    if (event == GGNavigationEvent.MENU_DOWN)
    {
      switch (state)
      {
        case DEPLOY:
          if (activeShip == null)
            return;
          activeShip.rotate();
          refresh();
          break;
      }
    }
    if (event == GGNavigationEvent.BACK_DOWN)
    {
      switch (state)
      {
        case DEPLOY:
          marker.setLocation(hideLocation);
          if (bp != null && bp.isConnected())  // Play again
          {
            if (isEnemyReady) // Enemy is ready, the game restarts
            {
              sendCommand(Command.restart);
              if (isGameLost)
              {
                state = State.MY_SHOOT;
                status.setText(msgMyShoot, DKGREEN);
              }
              else
              {
                state = State.ENEMY_SHOOT;
                status.setText(msgEnemyShoot, RED);
              }
            }
            else  // Enemy is not yet ready, we signal that we are ready to play
            {
              status.setText("Enemy not yet ready. Waiting..", DKGREEN);
              sendCommand(Command.ready);
            }
          }
          else
          {
            state = State.CONNECTING;
            Monitor.wakeUp();  // Wake up main thread
          }
          break;
        case GAME_OVER:  // Play again
          state = State.DEPLOY;
          init();
          break;
      }
    }
  }

  private void sendCommand(int command)
  {
    int[] data =
    {
      command, 0
    };
    bp.sendDataBlock(data);  // Signal enemy we are ready
  }

  // Signal to server that we quit
  public void onPause()
  {
    if (bp != null && bp.isConnected())
    {
      sendCommand(Command.disconnect);
      bp.releaseConnection();
      state = State.CONNECTION_LOST;
    }
    super.onPause();
  }

}
