// Ship.java

package ch.aplu.btbattleship;

import ch.aplu.android.*;
import android.graphics.*;

public abstract class Ship extends Actor
{ 
  protected int size;
  protected Location[] cells;   // Current occupied locations
  private boolean[] isHit;

  public Ship(String img, int size)
  {
    super(true, img); // Rotatable
    this.size = size;
    cells = new Location[size];
    isHit = new boolean[size];
  }

  public void reset()
  {
    Location loc = getLocation();
    setLocationOffset(getOffset());
    for (int i = 0; i < size; i++)
    {
      cells[i] = new Location(loc.x + i, loc.y);
      isHit[i] = false;
    }
  }

  protected void updateCells(Location loc)
  {
    int dir = getIntDirection();
    for (int i = 0; i < size; i++)
    {
      if (dir == 0)
        cells[i] = new Location(loc.x + i, loc.y);
      if (dir == 90)
        cells[i] = new Location(loc.x, loc.y + i);
      if (dir == 180)
        cells[i] = new Location(loc.x - i, loc.y);
      if (dir == 270)
        cells[i] = new Location(loc.x, loc.y - i);
    }
  }

  protected boolean rotate()
  // Returns true, if can rotate
  {
    for (int angle = 90; angle <= 270; angle += 90)
    {
      if (advance(getLocation(), angle))
        return true;
    }
    return false;
  }

  protected boolean advance(Location loc, int angle)
  // Try to advance to new location/direction
  // Return false, if outside grid or overlap with another ship
  {
    turn(angle);
    
    boolean ok = true;
    // Save current cell locations
    Location[] currentCells = new Location[size];
    for (int i = 0; i < size; i++)
      currentCells[i] = cells[i].clone();

    // Update cell locations
    updateCells(loc);

    // Check if all cells are in grid
    for (int i = 0; i < size; i++)
    {
      if (!gameGrid.isInGrid(cells[i]))
        ok = false;
    }

    // If ok, check for overlap
    if (ok)
    {
      for (Actor a : gameGrid.getActors(Ship.class))
      {
        if (a != this && overlap((Ship)a))
          ok = false;
      }
    }

    // If failed, restore old cells and old direction
    if (!ok)
    {
      for (int i = 0; i < size; i++)
        cells[i] = currentCells[i].clone();
      turn(-angle);
    }
    
    if (ok)
      setLocationOffset(getOffset());
    return ok;
  }

  private boolean overlap(Ship a)
  // Check if current ship overlaps with given ship a
  {
    for (int i = 0; i < size; i++)
      for (int k = 0; k < a.size; k++)
        if (cells[i].equals(a.cells[k]))
          return true;
    return false;
  }

  protected String hit(Location loc)
  {
    String reply = null;
    for (int i = 0; i < size; i++)
    {
      if (cells[i].equals(loc))
      {
        isHit[i] = true;
        gameGrid.addActor(new Fire()    , loc);
        int status = getStatus();
        if (status == 0)
          reply = "hit";
        if (status == 1)
          reply = "sunk";
        if (status == 2)
          reply = "allSunk";
      }
    }
    if (reply == null)
      reply = "miss";
    return reply;
  }

  private int getStatus()
  // 0: hit
  // 1: ship sunk
  // 2: all ships sunk
  {
    boolean allHit = true;
    for (int i = 0; i < size; i++)
      allHit = allHit && isHit[i];  // false, if one or more locs are not hit
    if (allHit)
    {
      removeSelf();
      for (int i = 0; i < size; i++)
        gameGrid.removeActorsAt(cells[i], Fire.class);
      if (gameGrid.getNumberOfActors(Ship.class) == 0)
        return 2;
      return 1;
    }
    return 0;
  }

  private Point getOffset()
  {
    int d = (gameGrid.getCellSize() / 2) * (size - 1);
    switch ((int)getDirection())
    {
      case 0:
        return new Point(d, 0);
      case 90:
        return new Point(0, d);
      case 180:
        return new Point(-d, 0);
      case 270:
        return new Point(0, -d);
    }
    return new Point(0, 0);
  }
  

}
