// Mandelbrot.java

package ch.aplu.mandelbrot;

import android.graphics.*;
import ch.aplu.android.*;

public class Mandelbrot extends GameGrid implements GGTouchListener
{
  private final double RADIUS_SQUARE = 2 * 2;
  private int maxIterations = 50;
  private GGPanel p;
  private Point start, end;
  private double ratio;
  private boolean isDrawing = false;

  public Mandelbrot()
  {
    super(true, null);
    setScreenOrientation(LANDSCAPE);
  }

  public void main()
  {
    showToast("Mandelbrot Fractal", true);
    ratio = (double)getNbHorzPix() / getNbVertPix();
    double yspan = 3.2 / ratio;
    draw(-2.2, 1.0, -yspan / 2, yspan / 2);
    addTouchListener(this, GGTouch.press | GGTouch.drag | GGTouch.release);
  }

  private void draw(double xmin, double xmax, double ymin, double ymax)
  {
    p = getPanel(Math.min(xmin, xmax), Math.max(xmin, xmax),
      Math.min(ymin, ymax), Math.max(ymin, ymax));
    p.setAutoRefreshEnabled(false);
    for (int xPixel = 0; xPixel < getNbHorzPix(); xPixel++)
    {
      for (int yPixel = 0; yPixel < getNbVertPix(); yPixel++)
      {
        double x_p = p.toUserX(xPixel);
        double y_p = p.toUserY(yPixel);
        int iterCount = getIterationCount(x_p, y_p);
        int drawColor = getIterationColor(iterCount);
        p.setPaintColor(drawColor);
        p.drawPoint(xPixel, yPixel);
      }
      if (xPixel % 100 == 0) // refresh every 50 columns
        refresh();
    }
    showToast("Drag to select subarea...", true);
    refresh();
    isDrawing = false;
  }

  private int getIterationColor(int it)
  {
    return Color.rgb((30 * it) % 256, (4 * it) % 256,
      255 - (30 * it) % 256);
  }

  private int getIterationCount(double px, double py)
  {
    if (isStable(px, py))
      return maxIterations;
    double x = 0, y = 0;
    int iter = 0;
    while (isInCircle(x, y) && iter < maxIterations)
    {
      double xt = x * x - y * y + px;
      double yt = 2 * x * y + py;
      x = xt;
      y = yt;
      iter++;
    }
    return iter;
  }

  private boolean isStable(double x, double y)
  {
    double p = Math.sqrt(Math.pow(x - 1 / 4f, 2) + y * y);
    return x < p - 2 * (p * p) + 1 / 4f || (x + 1) * (x + 1) + y * y < 1 / 16f;
  }

  private boolean isInCircle(double x, double y)
  {
    return x * x + y * y < RADIUS_SQUARE;
  }

  public boolean touchEvent(GGTouch mouse)
  {
    if (isDrawing)
      return true;
    Location l = toLocation(mouse.getX(), mouse.getY());
    switch (mouse.getEvent())
    {
      case GGTouch.press:
        start = new Point(l.x, l.y);
        end = new Point(l.x, l.y);
        p.setPaintColor(Color.GREEN);
        p.setLineWidth(4);
        p.setXORMode();
        break;
      case GGTouch.drag:
        p.drawRectangle(start, end, ratio, false); // erase old
        end = new Point(l.x, l.y);
        p.drawRectangle(start, end, ratio, false); // draw new
        refresh();
        break;
      case GGTouch.release:
        end = new Point(l.x, l.y);
        // Don't draw if zoom area too small
        if (Math.abs(end.x - start.x) < 50 || Math.abs(end.y - start.y) < 50)
        {
          showToast("Subarea too small");
          p.drawRectangle(start, end, ratio, false); // erase old
          refresh();
          return true;
        }
        p.setPaintMode();
        p.setLineWidth(0);
        end = p.drawRectangle(start, end, ratio, false);  // draw new
        refresh();
        isDrawing = true;
        new Thread()
        {
          public void run()
          {
            showToast("Please wait...");
            delay(2000);
            draw(p.toUserX(start.x), p.toUserX(end.x),
              p.toUserY(start.y), p.toUserY(end.y));
          }
        }.start();
        
        break;
    }
    return true;
  }
}
