// Vehicle.java
// Remote control of NXT brick

package ch.aplu.nxtvehicle;

import ch.aplu.android.*;
import android.graphics.Color;
import ch.aplu.android.nxt.*;

public class NxtVehicle extends GameGrid
  implements GGPushButtonListener, GGTouchListener, NxtConnectionListener
{
  enum State
  {
    STOPPED, FORWARD, BACKWARD
  };

  private State state = State.STOPPED;
  private NxtRobot robot;
  private Gear gear;
  private GGPushButton forward;
  private GGPushButton backward;
  private GGPushButton stop;
  private GGPushButton slower;
  private GGPushButton faster;
  private Actor info;
  private Actor steeringWheel;
  private GGTextField speedIndicator;
  private double steeringDir = 0;
  private Location steeringWheelLoc;
  private final int startSpeed = 10;
  private int speed;
  private final int speedStep = 2;
  private int oldSpeed = 1000;
  private static final int s = 300; // Screen size

  public NxtVehicle()
  {
    super(s, s, 1);
    setScreenOrientation(PORTRAIT);
  }

  public void main()
  {
    steeringWheelLoc = new Location(s / 2, 120);
    getBg().clear(Color.BLUE);
    addButtons();
     // speedIndicator refreshs manually:
    speedIndicator = 
      new GGTextField("0 km/h", new Location(s / 2 - 32, 220), false);
    speedIndicator.show(); 
    refresh();
    robot = new NxtRobot(this);
    robot.setVerbose(false);
    if (robot.connect() != NxtRobot.ConnectState.CONNECTED)
    {
      info.show(2);  // Connection failed
      refresh();
      return;
    }
    info.show(1);  // Connection established
    refresh();
    gear = new Gear();
    robot.addPart(gear);
    gear.setSpeed(speed);
    enableButtons();
    addTouchListener(this, GGTouch.press | GGTouch.drag);
    robot.addNxtConnectionListener(this);
  }

  private void addButtons()
  {
    steeringWheel = new Actor(true, "steeringwheel");
    addActor(steeringWheel, steeringWheelLoc);

    info = new Actor("info", 3);
    addActor(info, new Location(s / 2, 20));

    backward = new GGPushButton("backward");
    addActor(backward, new Location(60, 210));

    forward = new GGPushButton("forward");
    addActor(forward, new Location(s - 60, 210));

    stop = new GGPushButton("stop");
    addActor(stop, new Location(s / 2, 270));

    slower = new GGPushButton("slower");
    addActor(slower, new Location(60, 270));

    faster = new GGPushButton("faster");
    addActor(faster, new Location(s - 60, 270));
  }

  private void enableButtons()
  {
    forward.addPushButtonListener(this);
    backward.addPushButtonListener(this);
    stop.addPushButtonListener(this);
    slower.addPushButtonListener(this);
    slower.setRepeatPeriod(500);
    faster.addPushButtonListener(this);
    faster.setRepeatPeriod(500);
  }

  public boolean touchEvent(GGTouch touch)
  {
    Location location = toLocationInGrid(touch.getX(), touch.getY());

    switch (touch.getEvent())
    {
      case GGTouch.press:
      case GGTouch.drag:
        if (location.y < 120)
        {
          GGVector vSteer = new GGVector(location.x, location.y);
          GGVector vCenter = new GGVector(steeringWheelLoc.x, steeringWheelLoc.y);
          GGVector vDir = vSteer.sub(vCenter);
          steeringDir = vDir.getDirection() - 1.5 * Math.PI;
          steeringWheel.setDirection(Math.toDegrees(steeringDir));
          setMovement();
          refresh();
          delay(100);  // Do not send Bluetooth commands to NXT to fast, they are
                       // queued
        }
        break;
    }

    return true;
  }

  private void setMovement()
  {
    if (speed != oldSpeed)
    {
      speedIndicator.setText(Math.abs(speed) + " km/h");
      oldSpeed = speed;
      refresh();
      gear.setSpeed(speed);
    }
    if (steeringDir == 0)
    {
      if (state == State.FORWARD)
        gear.forward();
      if (state == State.BACKWARD)
        gear.backward();
    }
    else
    {
      double radius = Math.abs(0.1 / Math.sin(steeringDir));
      if (steeringDir > 0)
      {
        if (state == State.FORWARD)
          gear.rightArc(radius);
        if (state == State.BACKWARD)
          gear.rightArc(-radius);
      }
      else
      {
        if (state == State.FORWARD)
          gear.leftArc(radius);
        if (state == State.BACKWARD)
          gear.leftArc(-radius);
      }
    }
  }

  public void onPause()
  {
    if (robot != null)
      robot.exit();
    super.onPause();
  }

  public void buttonPressed(GGPushButton button)
  {
    if (!robot.isConnected())
    {
      showToast("Robot not connected");
      return;
    }

    if (button == forward)
    {  
      if (state == State.STOPPED)
        speed = startSpeed;
      state = State.FORWARD;
    }  
    if (button == backward)
    {  
      if (state == State.STOPPED)
        speed = startSpeed;
      state = State.BACKWARD;
    }  
    if (button == faster && state != State.STOPPED)
    {
      if (speed < 100)
      {
        speed += speedStep;
        gear.setSpeed(speed);
      }
    }
    if (button == slower && state != State.STOPPED)
    {
      if (speed > 0)
      {
        speed -= speedStep;
        gear.setSpeed(speed);
      }
    }
    if (button == stop)
    {
      state = State.STOPPED;
      speed = 0;
      oldSpeed = 1000;
      gear.stop();
    }
    setMovement();
  }

  public void buttonReleased(GGPushButton button)
  {
  }

  public void buttonClicked(GGPushButton button)
  {
  }

  public void buttonRepeated(GGPushButton button)
  {
    if (button == faster && state != State.STOPPED)
    {
      if (speed < 100)
      {
        speed += speedStep;
        gear.setSpeed(speed);
        setMovement();
      }
    }
    if (button == slower && state != State.STOPPED)
    {
      if (speed > 0)
      {
        speed -= speedStep;
        gear.setSpeed(speed);
      }
      setMovement();
    }
  }

  public void notifyConnection(boolean connected)
  {
    if (!connected)
    {
      info.show(2);
      refresh();
    }
  }

}
