// SensorHorizon.java
// Artificial horizon skeleton
// Normal vertical instrument position

package ch.aplu.sensor.horizon;

import android.graphics.Color;
import android.hardware.*;
import ch.aplu.android.*;
import static java.lang.Math.*;

public class SensorHorizon extends GameGrid
  implements GGSensorListener
{
  private float[] R = new float[9];
  private float[] I = new float[9];
  private float[] acc = new float[3];
  private float[] mag = new float[3];
  private final double r = 100;
  private GGStatusBar status;
  private GGPanel p;
  private float aircraftPitch;
  private float aircraftRoll;

  public SensorHorizon()
  {
    super(WHITE);
    status = addStatusBar(30);
    setScreenOrientation(LANDSCAPE);
  }

  public void main()
  {
    setWakeLockEnabled(true);
    p = getPanel(-110, 110, -110, 110);
    p.setAutoRefreshEnabled(false);

    GGSensor accelerometer = new GGSensor(this, Sensor.TYPE_ACCELEROMETER,
      SensorManager.SENSOR_DELAY_FASTEST);
    accelerometer.addSensorListener(this);
    accelerometer.setLowPassFilter(10, 5);

    GGSensor magnetometer = new GGSensor(this, Sensor.TYPE_MAGNETIC_FIELD,
      SensorManager.SENSOR_DELAY_FASTEST);
    magnetometer.addSensorListener(this);
    magnetometer.setLowPassFilter(10, 5);
    
    while (true)
    {
      p.clear(Color.rgb(10, 40, 10));
      drawDisplay();
      double b = aircraftPitch;
      double m = tan(toRadians(aircraftRoll));

      double x1 = (-b * m + sqrt((m * m + 1) * r * r - b * b)) / (m * m + 1);
      double y1 = m * x1 + b;
      double x2 = (-b * m - sqrt((m * m + 1) * r * r - b * b)) / (m * m + 1);
      double y2 = m * x2 + b;
      p.line(x1, y1, x2, y2);
      status.setText(String.format("Pitch:% 4.1f  Roll:% 4.1f",
        aircraftPitch, aircraftRoll));
      refresh();
      delay(50);
    }
  }

  public void sensorChanged(float[] values, int sensorType)
  {
    if (sensorType == Sensor.TYPE_ACCELEROMETER)
    {
      for (int i = 0; i < 3; i++)
        acc[i] = values[i];
    }

    if (sensorType == Sensor.TYPE_MAGNETIC_FIELD)
    {
      for (int i = 0; i < 3; i++)
        mag[i] = values[i];
    }

    boolean success =
      SensorManager.getRotationMatrix(R, I, acc, mag);
    if (!success)
      return;

    float[] zArrow =
    {
      0, 0, 1
    };
    float[] zArrow_world = GGSensor.deviceToWorld(R, zArrow);
    aircraftPitch = 90 - (float)toDegrees(Math.acos(zArrow_world[2]));

    float[] xArrow =
    {
      1, 0, 0
    };
    float[] xArrow_world = GGSensor.deviceToWorld(R, xArrow);

    float[] yArrow =
    {
      0, 1, 0
    };
    float[] yArrow_world = GGSensor.deviceToWorld(R, yArrow);

    if (isOrientationTablet())
      aircraftRoll = (float)toDegrees(Math.acos(xArrow_world[2])) - 90;
    else
      aircraftRoll = 90 - (float)toDegrees(Math.acos(yArrow_world[2]));
  }

  private void drawDisplay()
  {
    PointD p1;
    PointD p2;
    p.circle(new PointD(0, 0), r, false);
    for (int a = 0; a < 360; a += 10)
    {
      p1 = new PointD((r - 10) * cos(toRadians(a)), (r - 10) * sin(toRadians(a)));
      p2 = new PointD(r * cos(toRadians(a)), r * sin(toRadians(a)));
      p.line(p1, p2);
    }

    for (int y = -5; y <= 5; y++)
    {
      p1 = new PointD(-20, 10 * y);
      p2 = new PointD(20, 10 * y);
      p.line(p1, p2);
    }

    p1 = new PointD(35, 3);
    p2 = new PointD(65, -3);
    p.rectangle(p1, p2, true);

    p1 = new PointD(-35, 3);
    p2 = new PointD(-65, -3);
    p.rectangle(p1, p2, true);

    p.circle(new PointD(0, 0), 5, true);
  }

}
