// Player.java

package ch.aplu.tictactoe;

import ch.aplu.tcp.*;
import ch.aplu.android.*;
import android.graphics.Color;

public class Player extends TcpAgent
  implements GGTouchListener
{
  private class MyTcpAgentAdapter extends TcpAgentAdapter
  {
    public void dataReceived(String source, int[] data)
    {
      switch (data[0])
      {
        case Command.IN_PROGRESS:
          board.status.setBackgroundColor(RED);
          board.status.setText("Game room occupied");
          break;
        case Command.TEAMMATE_QUIT:
          board.status.setBackgroundColor(RED);
          board.status.setText("Partner quit. Waiting for a partner");
          board.clear();
          isMyTurn = false;
          isOver = false;
          break;
        case Command.TEAMMATE_WAIT:
          board.status.setBackgroundColor(PURPLE);
          board.status.setText("Waiting for a partner");
          break;
        case Command.MARK_X:
          mySign = 'X';
          teammateSign = 'O';
          guide = new Guide('X');
          break;
        case Command.MARK_O:
          mySign = 'O';
          teammateSign = 'X';
          guide = new Guide('O');
          break;
        case Command.START_FIRST:
          board.clear();
          board.addActor(guide, hideLoc);
          board.status.setBackgroundColor(GREEN);
          board.status.setText("Click to put your mark");
          isOver = false;
          isMyTurn = true;
          break;
        case Command.START_SECOND:
          board.clear();
          board.addActor(guide, hideLoc);
          board.status.setBackgroundColor(RED);
          board.status.setText("Wait for partner's move");
          isOver = false;
          isMyTurn = false;
          break;
        case Command.MOVE:
          Location loc = new Location(data[1], data[2]);
          board.addActor(new Mark(teammateSign), loc);
          board.status.setBackgroundColor(GREEN);
          board.status.setText("It's your move");
          isMyTurn = true;
          break;
        case Command.WON:
          isOver = true;
          board.status.setBackgroundColor(Color.BLUE);
          board.status.setText("You won. Click to restart");
          break;
        case Command.LOST:
          isOver = true;
          board.status.setBackgroundColor(Color.BLUE);
          board.status.setText("You lost. Click to restart");
          break;
        case Command.TIE:
          isOver = true;
          board.status.setBackgroundColor(Color.BLUE);
          board.status.setText("It is a draw. Click to restart");
          break;
      }
      if (isOver)
      {
        isMyTurn = false;
        board.addActor(new Actor("gameover"), new Location(1, 1));
      }
    }

    public void notifyBridgeConnection(boolean connected)
    {
      if (!connected)
      {
        board.status.setBackgroundColor(RED);
        board.status.setText("Game server stopped");
        disconnect();
      }
    }

  }

  private static final String nickname = "TicTacToePlayer";
  private final int timeout = 10000; // Connection timeout in ms
  private volatile boolean isMyTurn = false;
  private volatile boolean isOver = false;
  private char mySign;
  private char teammateSign;
  private Guide guide = null;
  private final TicTacToe board;
  private final Location hideLoc = new Location(-1, -1);
  private int PURPLE = Color.rgb(120, 0, 113);
  private int GREEN = Color.rgb(0, 120, 0);
  private int RED = Color.rgb(120, 0, 0);

  public Player(TicTacToe ttt, String sessionID, String bridgeName)
  {
    super(sessionID, bridgeName);
    board = ttt;
    board.addTouchListener(this, GGTouch.click | GGTouch.release | GGTouch.drag);
    addTcpAgentListener(new MyTcpAgentAdapter());
    board.status.setBackgroundColor(PURPLE);
    board.status.setText("Connecting to relay...");
    if (connectToRelay(nickname, timeout).isEmpty())
    {
      board.status.setBackgroundColor(RED);
      board.status.setText("Connection to relay failed");
      return;
    }

    if (!isBridgeConnected())
    {
      board.status.setBackgroundColor(RED);
      board.status.setText("Connected, but no game server");
      disconnect();
      return;
    }
  }

  public boolean touchEvent(GGTouch touch)
  {
    if (isOver && touch.getEvent() == GGTouch.click)
    {
      isOver = false;
      board.clear();
      board.status.setBackgroundColor(PURPLE);
      board.status.setText("Restart requested. Wait for partner");
      send("", Command.RESTART);
      return true;
    }

    if (isMyTurn)
    {
      Location loc = board.toLocation(touch.getX(), touch.getY());
      if (!board.isInGrid(loc))
        return true;
      switch (touch.getEvent())
      {
        case GGTouch.drag:
          if (!guide.getLocation().equals(loc))
          {
            if (board.getOneActorAt(loc, Mark.class) == null)
              guide.setLocation(loc);
            else
              guide.setLocation(hideLoc);
          }
          break;

        case GGTouch.release:
          if (board.getOneActorAt(loc, Mark.class) == null)
          {
            send("", Command.MOVE, loc.x, loc.y, mySign == 'O' ? 0 : 1);
            isMyTurn = false;
            board.status.setBackgroundColor(RED);
            board.status.setText("Wait for partner's move.");
            board.addActor(new Mark(mySign), loc);
          }
          break;
      }
    }
    return true;
  }

}
