// TicTacToeServer.java

package ch.aplu.tictactoeserver;

import ch.aplu.android.*;
import ch.aplu.tcp.*;
import java.util.ArrayList;

public class TicTacToeServer extends GameGrid 
{
  private final String VERSION = "1.1";
  private final String sessionIDPrefix = "TicTacToe_67%&%2134";
  private TcpBridge tcpBridge;
  private static final String bridgeName = "TicTacToeServer";
  // players.get(0) has mark 'X', players.get(1) has mark 'O'
  private ArrayList<String> players = new ArrayList<String>();
  private GGConsole c;
  private char[][] gameState = new char[3][3];
  private int startRequest;
  private String[] startingGrid = new String[2];

  public void main()
  {
    c = GGConsole.init();
    String sessionID = getSessionID();
    if (sessionID == null)
    {
      c.println("Server canceled");
      return;
    }
    tcpBridge = new TcpBridge(sessionID, bridgeName);
    tcpBridge.addTcpBridgeListener(new TcpBridgeAdapter()
    {
      public void pipeRequest(String source, String destination, int[] data)
      {
        showPipeRequest(source, data);
        String teammate = source.contains("(1)")
          ? TcpTools.stripNickname(source) : (source + "(1)");

        switch (data[0])
        {
          case Command.MOVE:
            char value = data[3] == 0 ? 'O' : 'X';
            gameState[data[1]][data[2]] = value;
            tcpBridge.send("", teammate, data);
            String[] playerState = getPlayerState();
            if (playerState == null)  // still in progress
            {
              if (isBoardFull())
              {
                c.println("Game over. It's a draw");
                tcpBridge.send("", players.get(0), Command.TIE);
                tcpBridge.send("", players.get(1), Command.TIE);
                // Exchange starting grid
                String temp = startingGrid[1];
                startingGrid[1] = startingGrid[0];
                startingGrid[0] = temp;
              }
            }
            else
            {
              c.println("Game over. Winner: " + playerState[0] + ". Loser: "
                + playerState[1]);
              tcpBridge.send("", playerState[0], Command.WON);
              tcpBridge.send("", playerState[1], Command.LOST);
              startingGrid[0] = playerState[1];  // Loser makes first move
              startingGrid[1] = playerState[0];
            }
            break;

          case Command.RESTART:
            startRequest++;
            if (startRequest == 2)
              initGame();
            break;
        }
      }

      public void notifyAgentConnection(String agentName, boolean connected)
      {
        c.println(agentName + (connected ? " connected" : " disconnected"));
        if (connected)
        {
          if (players.size() >= 2)
          {
            tcpBridge.send("", agentName, Command.IN_PROGRESS);
            return;
          }

          players.add(agentName);

          if (players.size() == 1)
          {
            tcpBridge.send("", players.get(0), Command.TEAMMATE_WAIT);
            c.println("Wait for teammate.");
          }

          if (players.size() == 2)
          {
            // First player logged in will start
            startingGrid[0] = players.get(0);
            startingGrid[1] = players.get(1);
            initGame();  
            c.println("Game started.");
          }
        }
        else  // disconnected
        {
          players.remove(agentName);
          if (players.size() == 1)
          {
            tcpBridge.send("", players.get(0), Command.TEAMMATE_QUIT);
            c.println("Teammate abandoned.");
          }
        }
      }
    });

    c.println("Connecting to relay...");
    if (tcpBridge.connectToRelay().size() > 1)
    {
      c.println("Connected. But game room ID in use.");
      tcpBridge.disconnect();
      c.println("Disconnected from relay.");
    }
    else
      c.println("Waiting for players...");
  }

  private boolean isBoardFull()
  {
    int nb = 0;
    for (int i = 0; i < 3; i++)
      for (int k = 0; k < 3; k++)
        if (gameState[i][k] != ' ')
          nb++;
    return nb == 9;
  }

  private String[] getPlayerState()
  {
    // Convert gameState into string pattern
    StringBuffer pattern = new StringBuffer();
    // Horizontal
    for (int i = 0; i < 3; i++)
    {
      for (int k = 0; k < 3; k++)
      {
        pattern.append(gameState[i][k]);
      }
      pattern.append(',');  // Separator
    }
    // Vertical
    for (int k = 0; k < 3; k++)
    {
      for (int i = 0; i < 3; i++)
      {
        pattern.append(gameState[i][k]);
      }
      pattern.append(',');
    }
    // Diagonal
    for (int i = 0; i < 3; i++)
      pattern.append(gameState[i][i]);
    pattern.append(',');
    for (int i = 0; i < 3; i++)
      pattern.append(gameState[i][2 - i]);

    c.println("Pattern: " + pattern);
    String[] result = new String[2];
    if (pattern.toString().contains("XXX"))
    {
      result[0] = players.get(0);
      result[1] = players.get(1);
      return result;
    }
    if (pattern.toString().contains("OOO"))
    {
      result[0] = players.get(1);
      result[1] = players.get(0);
      return result;
    }
    return null;
  }

  private void initGame()
  {
    for (int i = 0; i < 3; i++)
      for (int k = 0; k < 3; k++)
        gameState[i][k] = ' ';
    startRequest = 0;
    tcpBridge.send("", players.get(0), Command.MARK_X);
    tcpBridge.send("", players.get(1), Command.MARK_O);
    // Send first to 'second' to make him ready to receive move requests
    tcpBridge.send("", startingGrid[1], Command.START_SECOND);
    TcpTools.delay(2000);
    tcpBridge.send("", startingGrid[0], Command.START_FIRST); 
  }

  private void showPipeRequest(String source, int[] data)
  {
    c.println("pipeRequest from: " + source);
    String text = "Data: ";
    for (Integer k : data)
      text += (k + " ");
    c.println(text);
  }
  
  private String getSessionID()
  {
    String roomID = "";
    while (roomID.length() < 3)
    {
      roomID = GGInputDialog.show(c.getActivity(), "TicTacToeServer V" + VERSION, 
        "Enter unique game room name (more than 2 characters):", "");
      if (roomID == null)  // Cancel
        return null;
      roomID = roomID.trim();
    }
    return sessionIDPrefix + roomID;
  }

}


